#include "listmgr.hpp"

DisplayListNode::DisplayListNode(Icon* item)
{
	entry = item;
	predecessor = successor = 0;
}


DisplayList::DisplayList()
{
	// create a null list
	first_item = 0;
	current_item = 0;
	list_length = 0;
}

void DisplayList::AddItem(Icon* item)
{
	// create a new node
	DisplayListNode* node = new DisplayListNode(item);
	if (!node)
	  // whoops -- no memory left
	  FatalListError("memory allocation failure");

	if (list_length == 0) {

	  // make the node the list's first element
	  first_item = node;

	  // connect the root to itself, so the list
	  // is circular
	  first_item->predecessor =
	      first_item->successor = first_item;
	}

	else if (list_length == 1) {
	  // add a second node to the list
	  first_item->predecessor = node;
	  first_item->successor = node;
	  node->successor = first_item;
	  node->predecessor = first_item;
	}

	else {

	  // splice the new node in right after
	  // the current one
	  node->predecessor = current_item;
	  node->successor = current_item->successor;
	  current_item->successor->predecessor = node;
	  current_item->successor = node;
	}

	// update the list variables
	current_item = node;
	list_length++;
}

void DisplayList::RemoveItem(Icon* item)
{
	// search for the item in the list,
	// starting from the beginning
	GotoBeginning();
	for (int i = 0; i < list_length; i++, NextItem() ) {

	  // see if the next icon in the display list
	  // is the one we're looking for
	  if (item == CurrentItem() ) {
	    // it is: remove it from the display list
	    DeleteCurrentNode();
	    return;
	  }
	}
}

void DisplayList::DeleteCurrentNode()
{
	// abort if no items in list
	if (current_item == 0)
	  return;

	// remove the current item from the list
	current_item->predecessor->successor =
	  current_item->successor;
	current_item->successor->predecessor =
	  current_item->predecessor;

	// delete the item and update the list
	delete current_item;
	current_item = current_item->successor;
	list_length--;
}

Icon* DisplayList::CurrentItem()
{
	// abort if no items in list
	if (current_item == 0)
	    FatalListError("list access error");

	return current_item->entry;
}

void DisplayList::GotoBeginning()
{
	current_item = first_item;
}


void DisplayList::GotoEnd()
{
	current_item = first_item->predecessor;
}


void DisplayList::FatalListError(char* string)
{
	printf("\nDisplay List: ");
	printf("%s\n", string);
	exit(1);
}

DisplayList::~DisplayList()
{
	GotoBeginning();

	while (list_length)
	  DeleteCurrentNode();
}
